//modified: 2002.06.09@21:17 by Alex

//Projekt: TI-Server-Log-Analyser

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, UnitMy, ExtCtrls, Math, ShellAPI;

type
  TForm1 = class(TForm)
    MemoOutput: TMemo;
    Panel1: TPanel;
    Label1: TLabel;
    EditLogFile: TEdit;
    CmdBrowse: TButton;
    OpenDialog: TOpenDialog;
    CmdAnalyse: TButton;
    CheckShowFreigabe: TCheckBox;
    CheckShowNOTEs: TCheckBox;
    CheckLeaveOutDROPTXEN: TCheckBox;
    CheckShowNoteDeleteAllBefore: TCheckBox;
    CheckShowNoteEmptyingRcvBuf: TCheckBox;
    CheckShowNoteRcvWait: TCheckBox;
    CheckShowNoteDeletingFrom: TCheckBox;
    CmdOpenLogFile: TButton;
    CmdOpenOutput: TButton;
    CheckShowEmptyPackets: TCheckBox;
    procedure CmdBrowseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure CmdAnalyseClick(Sender: TObject);
    procedure CheckShowFreigabeClick(Sender: TObject);
    procedure CmdOpenLogFileClick(Sender: TObject);
    procedure CheckShowNOTEsClick(Sender: TObject);
    procedure CmdOpenOutputClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1  : TForm1;
  started: boolean;

implementation

{$R *.DFM}

procedure TForm1.FormShow(Sender: TObject);
//Einstellungen laden
var
  MyRegistry : TMyRegistry;
  i          : integer;
begin
  if started then exit;
  MyRegistry:=TMyRegistry.Create;
  MyRegistry.RootKey:=HKEY_CURRENT_USER;
  MyRegistry.OpenKey('Software\85\TI-Server-Log-Analyser\',true);
  with Form1 do
    for i:=0 to ComponentCount-1 do begin
      if (Components[i] is TEdit) then
        (Components[i] as TEdit).text:=MyRegistry.ReadStringD(Components[i].Name,(Components[i] as TEdit).text)
      else if (Components[i] is TRadioButton) then
        (Components[i] as TRadioButton).Checked:=MyRegistry.ReadBoolean(Components[i].Name,(Components[i] as TRadioButton).Checked)
      else if (Components[i] is TListBox) then
        (Components[i] as TListBox).ItemIndex:=MyRegistry.ReadIntegerD(Components[i].Name,(Components[i] as TListBox).ItemIndex)
      else if (Components[i] is TCheckBox) then
        (Components[i] as TCheckBox).Checked:=MyRegistry.ReadBoolean(Components[i].Name,(Components[i] as TCheckBox).Checked);
    end;
  started:=true;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
//Einstellungen speichern
var
  MyRegistry : TMyRegistry;
  i          : integer;
begin
  MyRegistry:=TMyRegistry.Create;
  MyRegistry.RootKey:=HKEY_CURRENT_USER;
  MyRegistry.OpenKey('Software\85\TI-Server-Log-Analyser\',true);
  with Form1 do
    for i:=0 to ComponentCount-1 do begin
      if (Components[i] is TEdit) then
        MyRegistry.WriteString(Components[i].Name,(Components[i] as TEdit).text)
      else if (Components[i] is TRadioButton) then
        MyRegistry.WriteBoolean(Components[i].Name,(Components[i] as TRadioButton).Checked)
      else if (Components[i] is TListBox) then
        MyRegistry.WriteInteger(Components[i].Name,(Components[i] as TListBox).ItemIndex)
      else if (Components[i] is TCheckBox) then
        MyRegistry.WriteBoolean(Components[i].Name,(Components[i] as TCheckBox).Checked);
      end;
  MyRegistry.Free;
end;

procedure TForm1.CmdBrowseClick(Sender: TObject);
//choose log file
begin
  if OpenDialog.Execute then EditLogFile.Text:=OpenDialog.FileName;
end;

procedure TForm1.CmdAnalyseClick(Sender: TObject);
//start analysis
var
  logfile : textfile;
  outfile : textfile;
  a,b,c   : string;
  output  : string;   //temp. storage for analysis
  line    : string;
  i,j     : integer;
  action  : boolean;  //something new?
  final   : string;   //this is put into the memo at the end of this proc
  timest  : string;
  anmeldefreigabe,sendefreigabe:boolean;
begin
  CmdAnalyse.Enabled:=false;
  if not FileExists(EditLogFile.Text) then begin
    ShowMessage('Datei "'+EditLogFile.Text+'" nicht gefunden.');
    CmdAnalyse.Enabled:=true;
    exit;
  end;
  try
    AssignFile(logfile,EditLogFile.Text);
    ReSet(logfile);
  except
    ShowMessage('Fehler beim ffnen der Log-Datei.');
    CmdAnalyse.Enabled:=true;
    exit;
  end;
  ReadLn(logfile,a);
  if pos('TI SERVER LOGFILE',a)=0 then
    if MessageDlg('Keine gltige TI-Server-Logdatei.',mtWarning,[mbIgnore,mbCancel],0)=mrCancel then begin
      CmdAnalyse.Enabled:=true;
      exit;
    end;
  final:='';
  output          := '';
  action          := false;
  sendefreigabe   := false;
  anmeldefreigabe := false;
  MemoOutput.Clear;
  //proc's main loop
  while not eof(logfile) do begin
    ReadLn(logfile,a);
    val(a,i,j);
    if j=1 then Continue; //ignore line if no number at beginning
    if (j>0) and (a[j]<>':') then Continue; //ignore line if no ':' after number
    //valid line in a, j points to ':'
    b:=copy(a,j+1,4);
    if b='====' then begin
      if action then final:=final+timest+':'#13#10+output+'[...]'#13#10;
      timest:=copy(a,1,j-1);
      output:='';
      action:=false;
    end else
    if (b='SEND') or (b='RCVD') then begin
      b:=DeCodeText(copy(a,j+8,length(a)));
      line:=format('%3.3d->%3.3d:',[ord(b[4]),ord(b[1])]);
      // examine content id
      case ord(b[3]) of
         0 : begin
               if not anmeldefreigabe then line:='FEHLER: Anmeldung hier nicht gltig.';
               line:=line+'Login von '+EnCodeText(copy(b,6,8))+#13#10;
               action:=true;
             end;
         1 : begin
               line:=line+'ACK'#13#10;
               action:=true;
             end;
         2 : begin
               line:=line+'Client abort von '+inttostr(ord(b[4]))+#13#10;
               action:=true;
             end;
         3 : begin
               line:=line+'Logout von '+inttostr(ord(b[4]))+#13#10;
               action:=true;
             end;
         4 : begin
               if CheckShowFreigabe.Checked then
                 line:=line+'Sendefreigabe'#13#10
               else
                 line:='';
               anmeldefreigabe:=false;
               sendefreigabe  :=true;
             end;
         5 : begin
               line:=line+'Fehler (NACK) von '+inttostr(ord(b[4]))+#13#10;
               action:=true;
             end;
         6 : begin
               if CheckShowFreigabe.Checked then
                 line:=line+'Anmeldefreigabe'#13#10
               else
                 line:='';
               anmeldefreigabe:=true;
               sendefreigabe  :=false;
             end;
         7 : begin
               line:=line+'Login Besttigung an '+EnCodeText(copy(b,6,8))+', Adresse:'+inttostr(ord(b[14]))+#13#10;
               action:=true;
             end;
         8 : begin
               line:=line+'Logout Besttigung an '+inttostr(ord(b[1]))+#13#10;
               action:=true;
             end;
         9 : begin
               line:=line+'Neue Adresse '+inttostr(ord(b[14]))+' fr '+EnCodeText(copy(b,6,8))+#13#10;
               action:=true;
             end;
        10 : begin
               action:=action or CheckShowEmptyPackets.Checked;
               if action then
                 line:=line+'Empty Packet'#13#10
               else
                 line:='';
             end;
        11 : begin
               if sendefreigabe then line:='FEHLER: Senden hier nicht gltig.';
               line:=line+'Plain Text:'+copy(b,6,length(b)-7)+#13#10;
               action:=true;
             end;
        14 : begin
               if sendefreigabe then line:='FEHLER: Senden hier nicht gltig.';
               c:=copy(b,9,17);
               line:=line+'Vollbildmen ('+copy(c,1,pos(#0,c)-1)+')'#13#10;
               action:=true;
             end;
        15 : begin
               if sendefreigabe then line:='FEHLER: Senden hier nicht gltig.';
               line:=line+'Einzeiliges Eingabefeld ('+copy(b,9,ord(b[8]))+')'#13#10;
               action:=true;
             end;
        32 : begin
               if not sendefreigabe then line:='FEHLER: Senden hier nicht gltig.';
               line:=line+'Antwort auf Vollbildmen (Punkt '+chr(ord(b[6])+48)+')'#13#10;
               action:=true;
             end;
        33 : begin
               if not sendefreigabe then line:='FEHLER: Senden hier nicht gltig.';
               line:=line+'Antwort auf Plain text (Key Code:'+inttostr(ord(b[6]))+')'#13#10;
               action:=true;
             end;
        36 : begin
               if not sendefreigabe then line:='FEHLER: Senden hier nicht gltig.';
               line:=line+'Antwort auf einzeiliges Eingabefeld ('+copy(b,8,ord(b[7]))+')'#13#10;
               action:=true;
             end;
        128..255 :
             begin
               if sendefreigabe then line:='FEHLER: Senden hier nicht gltig.';
               line:=line+format('Fortgesetztes Multipacket (ID %d Teil %d)'#13#10,[ord(b[3]) xor $80,ord(b[5])]);
               action:=true;
             end;
        else begin
               line:=line+'Unbekannte ID '+inttostr(ord(b[3]))+' in '+EnCodeText(b)+#13#10;
               action:=true;
             end;
      end; //of case
      output:=output+line;
      line:='';
    end else
    if (b='DROP') then begin
      if (CheckLeaveOutDROPTXEN.Checked) and (copy(a,j+1,11)='DROP : TXEN') then begin
        output:=output+copy(a,j+1,length(a))+#13#10;
        action:=true;
      end;
    end else
    if (b='NOTE') and (CheckShowNOTEs.Checked) then begin
      if (pos('receive timeout',a)>0) then begin
        if sendefreigabe then begin
          output:=output+'FEHLER: TI antwortet nicht'#13#10;
          action:=true;
        end;
      end else
      if (pos('Delete all before',a)>0) then begin
        if CheckShowNoteDeleteAllBefore.Checked then begin
          output:=output+copy(a,j+1,length(a))+#13#10;
          action:=true;
        end;
      end else
      if (pos('emptying rcvbuffer',a)>0) then begin
        if CheckShowNoteEmptyingRcvBuf.Checked then begin
          output:=output+copy(a,j+1,length(a))+#13#10;
          action:=true;
        end;
      end else
      if (pos('receive-wait',a)>0) then begin
        if CheckShowNoteRcvWait.Checked then begin
          output:=output+copy(a,j+1,length(a))+#13#10;
          action:=true;
        end;
      end else
      if (pos('deleting from',a)>0) then begin
        if CheckShowNoteDeletingFrom.Checked then begin
          output:=output+copy(a,j+1,length(a))+#13#10;
          action:=true;
        end;
      end else
        output:=output+copy(a,j+1,length(a))+#13#10;
    end; //of "if (b='xxxx') then begin ... end else if (b='xxxx') ..."
  end;
  MemoOutput.Text:=final;
  CloseFile(logfile);
  try
    AssignFile(outfile,ExtractFilePath(EditLogFile.Text)+'output.txt');
    ReWrite(outfile);
    Write(outfile,final);
    CloseFile(outfile);
  except
    ShowMessage('Konnte "'+ExtractFilePath(EditLogFile.Text)+'output.txt" nicht schreiben.');
  end;
  if MemoOutput.Text='' then
    ShowMessage('Die Ausgabe ist in der Datei "'+ExtractFilePath(EditLogFile.Text)+'output.txt" verfgbar.');
  CmdAnalyse.Enabled:=true;
end;

procedure TForm1.CheckShowFreigabeClick(Sender: TObject);
begin
  CmdAnalyseClick(nil);
end;

procedure TForm1.CmdOpenLogFileClick(Sender: TObject);
begin
  if FileExists(EditLogFile.Text) then
    ShellExecute(Hwnd(self),lpctstr('open'),lpctstr(EditLogFile.Text),lpctstr(''),lpctstr(''),sw_show);
end;

procedure TForm1.CheckShowNOTEsClick(Sender: TObject);
begin
  CheckShowNoteDeleteAllBefore.Enabled:=CheckShowNOTEs.Checked;
  CheckShowNoteEmptyingRcvBuf.Enabled:=CheckShowNOTEs.Checked;
  CheckShowNoteRcvWait.Enabled:=CheckShowNOTEs.Checked;
end;

procedure TForm1.CmdOpenOutputClick(Sender: TObject);
begin
  if not FileExists(ExtractFilePath(EditLogFile.Text)+'output.txt') then
    ShowMessage('"'+ExtractFilePath(EditLogFile.Text)+'output.txt" existiert nicht. Bitte Analyse neu starten.')
  else
    ShellExecute(Hwnd(self),lpctstr('open'),lpctstr(ExtractFilePath(EditLogFile.Text)+'output.txt'),lpctstr(''),lpctstr(''),sw_show);
end;

end.
